#include <stdio.h>
#include <Wire.h>
#include <LiquidCrystal_I2C.h>

#define CMD_Play_Next                                         0x01
#define CMD_Play_Previous                                     0x02
#define CMD_Specify_playback_of_a_track                       0x03
#define CMD_Increase_volume                                   0x04
#define CMD_Decrease_volume                                   0x05
#define CMD_Specify_volume                                    0x06
#define CMD_Specify_EQ                                        0x07
#define CMD_Specify_single_repeat_playback                    0x08
#define CMD_Specify_playback_of_a_device                      0x09
#define CMD_Set_Sleep                                         0x0A
#define CMD_Reset                                             0x0C
#define CMD_Play                                              0x0D
#define CMD_Pause                                             0x0E
#define CMD_Specify_playback_a_track_in_a_folder              0x0F
#define CMD_Audio_amplification_setting                       0x10
#define CMD_Set_all_repeat_playback                           0x11
#define CMD_Specify_playback_of_folder_named_MP3              0x12
#define CMD_Inter_cut_an_advertisement                        0x13
#define CMD_Specify_playback_a_track_in_a_folder_3000         0x14
#define CMD_Stop_playing_intercut                            0x15
#define CMD_Stop                                              0x16
#define CMD_Specify_repeat_playback_of_a_folder               0x17
#define CMD_Set_random_playback                               0x18
#define CMD_Set_repeat_playback_of_current_track              0x19
#define CMD_Set_DAC                                           0x1A
#define CMD_Query_current_online_storage_device               0x3F
#define CMD_Module_returns_an_error_data_with_this_command    0x40
#define CMD_Module_reports_a_feedback_with_this_command       0x41
#define CMD_Query_current_status                              0x42
#define CMD_Query_current_volume                              0x43
#define CMD_Query_current_EQ                                  0x44
#define CMD_Query_total_file_numbers_of_USB_flash_disk        0x47
#define CMD_Query_total_file_numbers_of_micro_SD_Card         0x48
#define CMD_Query_current_track_of_USB_flash_disk             0x4B
#define CMD_Query_current_track_of_micro_SD_Card              0x4C
#define CMD_Query_total_file_numbers_of_a_folder              0x4E
#define CMD_Query_total_folder_numbers_of_the_storage_device  0x4F

#define TOTAL_FRAME_LEN 10
#define PWM_PIN 2

#define MOTOR_OFF_TH  100
#define MOTOR_1_TH    280
#define MOTOR_2_TH    460
#define MOTOR_3_TH    640
#define MOTOR_4_TH    820
#define MOTOR_5_TH    1000
#define HYST          10

#define MOTOR_OFF       0
#define MOTOR_1         1
#define MOTOR_2         2
#define MOTOR_3         3
#define MOTOR_4         4
#define MOTOR_5         5
#define MOTOR_STARTING  6
#define MOTOR_STOPPING  7

typedef union {
    uint8_t bytes[TOTAL_FRAME_LEN];
    struct {
        uint8_t _start_byte;
        uint8_t _version;
        uint8_t _length;
        uint8_t CMD;
        uint8_t feedback;
        uint8_t DATA_HI;
        uint8_t DATA_LO;
        uint16_t _chksum;
        uint8_t _end_byte;
    };
} T_Frame;

T_Frame send_frame, receive_frame;
bool sound_played = LOW;
uint8_t motor_state = MOTOR_OFF;

LiquidCrystal_I2C lcd(0x27,16,2);  // set the LCD address to 0x27 for a 16 chars and 2 line display


///////////////////////////////////////////////////////////////////
// Ajoute un octet à la trame en cours de réception
///////////////////////////////////////////////////////////////////
void AddByteToFrame (uint8_t addByte) {
  int i = 0;
  for (i=0; i<TOTAL_FRAME_LEN-1; i++) receive_frame.bytes[i] = receive_frame.bytes[i+1];
  receive_frame.bytes[TOTAL_FRAME_LEN-1] = addByte;
}

///////////////////////////////////////////////////////////////////
// Calcul le checksum de la trame
///////////////////////////////////////////////////////////////////

int ComputeChecksum(T_Frame * frame) {
  uint8_t i = 0;

  frame->_chksum = 0;
  for (i=0; i<frame->_length; i++) {
    frame->_chksum += frame->bytes[i+1];
  }
  
  frame->_chksum = -(frame->_chksum);
  frame->_chksum = __builtin_bswap16(frame->_chksum);
    
  return 0;
}


///////////////////////////////////////////////////////////////////
// Envoie la trame au module MP3
///////////////////////////////////////////////////////////////////

void SendMp3Command(uint8_t data_lo, uint8_t data_hi, uint8_t command) {
  send_frame.CMD = command;
  send_frame.DATA_HI = data_lo;
  send_frame.DATA_LO = data_hi;
  ComputeChecksum(&send_frame);
  Serial.write(send_frame.bytes, TOTAL_FRAME_LEN);
}


///////////////////////////////////////////////////////////////////
// Reçoie les trames du module MP3
///////////////////////////////////////////////////////////////////

bool ReceiveMp3Command() {
  bool new_data = LOW;
  uint16_t rec_chksum;
  
  while(Serial.available()) { // there is data in the Serial buffer
    AddByteToFrame(Serial.read());
    new_data = HIGH;
  }
  if (new_data) {
    if (receive_frame._start_byte == 0x7E && receive_frame._end_byte == 0xEF){
      rec_chksum = receive_frame._chksum;
      ComputeChecksum(&receive_frame);
      if (receive_frame._chksum == rec_chksum) {
        // Trame valide reçue 
        // fin de lecture: 7EFF 063D 0000 xxFE B6EF 
        if (receive_frame.CMD == 0x3D) sound_played = HIGH;
      }
    }
  }
}

///////////////////////////////////////////////////////////////////
// Joue un fichier son
///////////////////////////////////////////////////////////////////
void PlaySound (uint8_t folderNum, uint8_t fileNum) {
  SendMp3Command(folderNum, fileNum, CMD_Specify_playback_a_track_in_a_folder);
}

///////////////////////////////////////////////////////////////////
// setup
///////////////////////////////////////////////////////////////////

void setup() {
  // put your setup code here, to run once:
  pinMode(13, OUTPUT);
  pinMode(PWM_PIN, INPUT);
  Serial.begin(9600);

  lcd.init();                      // initialize the lcd 
  lcd.init();
  // Print a message to the LCD.
  lcd.backlight();
  lcd.setCursor(0,0);
  lcd.print("Bruiteur avion");
  lcd.setCursor(0,1);
  lcd.print("v 1.1.0");

  send_frame._start_byte = 0x7E;
  send_frame._end_byte   = 0xEF;
  send_frame._version    = 0xFF;
  send_frame._length     = 6;
  send_frame.feedback    = 0;

  sound_played = LOW;
}


///////////////////////////////////////////////////////////////////
// mainloop
///////////////////////////////////////////////////////////////////

int pwm_value;
int toogle = 0;
uint8_t motor_state_old;
bool resetSoundPlayed = LOW;  
bool newState = LOW;

void loop() {  
  // put your main code here, to run repeatedly:
  digitalWrite(13, resetSoundPlayed);  
  pwm_value = pulseIn(PWM_PIN, HIGH, 44000);
  pwm_value -= 1000;

  ReceiveMp3Command();  

  motor_state_old = motor_state;

  if (sound_played) resetSoundPlayed = HIGH;

  // Motor state machine
  switch (motor_state) {
    case MOTOR_OFF :
      if (pwm_value > MOTOR_OFF_TH) {
        motor_state = MOTOR_STARTING;
        PlaySound (1, 1);
      }
      break;
    case MOTOR_STARTING :
        if (resetSoundPlayed) {
          motor_state = MOTOR_1;
          PlaySound (1, 2);
        }
      break;
    case MOTOR_STOPPING :
      if (resetSoundPlayed) {
          motor_state = MOTOR_OFF;
        }
      break;
    case MOTOR_1 :           
      //if (resetSoundPlayed) {
        if (pwm_value < MOTOR_OFF_TH-HYST) {
          motor_state = MOTOR_STOPPING;
          PlaySound (1, 7);
        }
        else if (pwm_value > MOTOR_1_TH+HYST) {
          motor_state = MOTOR_2;
          PlaySound (1, 3);
        }
      /*}
      else */if(toogle == 0) SendMp3Command(0, 0, CMD_Set_repeat_playback_of_current_track); 
      break;
    case MOTOR_2 : 
      //if (resetSoundPlayed) {
        if (pwm_value < MOTOR_1_TH-HYST) {
          motor_state = MOTOR_1;
          PlaySound (1, 2);
        }
        else if (pwm_value > MOTOR_2_TH+HYST) {
          motor_state = MOTOR_3;
          PlaySound (1, 4);
        }
      /*}
      else */if(toogle == 0) SendMp3Command(0, 0, CMD_Set_repeat_playback_of_current_track); 
      break;
    case MOTOR_3 :
      //if (resetSoundPlayed) {
        if (pwm_value < MOTOR_2_TH-HYST) {
          motor_state = MOTOR_2;
          PlaySound (1, 3);
        }
        else if (pwm_value > MOTOR_3_TH+HYST) {
          motor_state = MOTOR_4;
          PlaySound (1, 5);
        }
      /*}
      else */if(toogle == 0) SendMp3Command(0, 0, CMD_Set_repeat_playback_of_current_track); 
      break;
    case MOTOR_4 :
      //if (resetSoundPlayed) {
        if (pwm_value < MOTOR_2_TH-HYST) {
          motor_state = MOTOR_3;
          PlaySound (1, 4);
        }
        else if (pwm_value > MOTOR_3_TH+HYST) {
          motor_state = MOTOR_5;
          PlaySound (1, 6);
        }
      /*}
      else */if(toogle == 0) SendMp3Command(0, 0, CMD_Set_repeat_playback_of_current_track); 
      break;
    case MOTOR_5 :
      //if (resetSoundPlayed) {
        if (pwm_value < MOTOR_3_TH-HYST) {
          motor_state = MOTOR_3;
          PlaySound (1, 5);
        }
      /*}
      else */if(toogle == 0) SendMp3Command(0, 0, CMD_Set_repeat_playback_of_current_track); 
      break;
    default :
      break;
  }  
  if (resetSoundPlayed) {
    resetSoundPlayed = LOW;
    sound_played = LOW;
  }
  
  if (motor_state != motor_state_old) newState = HIGH;
  else newState = LOW;
  toogle++;
  if (toogle >= 20) 
  {
    toogle = 0;
    lcd.clear();    
    lcd.setCursor(0, 0);
    lcd.print("motor_state="); lcd.print(motor_state, DEC);
    lcd.setCursor(0, 1);
    lcd.print("pwm_value="); lcd.print(pwm_value, DEC);
  }

  
  //Serial.println(motor_state);
}
